<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once(BASEPATH.'libraries/Model.php');
require_once(APPPATH.'classes/dataobjects/TransactionRequestData.class.php');
/**
 * @abstract
 */
abstract class HttpRequestParserModelBase extends Model
{
	/**
	 * @var string リクエストデータを保持する DataObjectBase 型のクラスの名前
	 * @abstract
	 */
	protected $_object_name = 'TransactionRequestData';
	/**
	 * @var array[] 妥当性検査情報の配列
	 * @abstract
	 */
	protected $_validation_rules = array();

	/**
	 * @var DataObjectBase リクエストデータを保持する DataObjectBase オブジェクト
	 */
	private $_object = null;
	/**
	 * @var string[] エラメッセージを格納した配列
	 */
	private $_errors = array();
	/**
	 * コンストラクタ
	 */
    public function __construct()
    {
        parent::__construct();
    }
	/**
	 * HTTP リクエストを解析し、データメンバを初期化後、妥当性を検査します。
	 *
	 * @param bool $isvalidate 妥当性を検査する場合は true を指定（デフォルト）
	 * @param string[] $exclude チェックから除外するフィールドの配列。
	 * @return bool 妥当性検査の結果、正常ならば true。検査省略時は必ず true
	 */
	public function parseRequest($isvalidate = true, $exclude = array())
	{
		// HTTP リクエストを解析し、データメンバを初期化
		$this->_parseHttpRequest();
		// データの妥当性を検証
		if($isvalidate)
			return $this->_validateField($exclude);
		else
			return true;
	}
    /**
     * HTTP リクエストを解析し、データメンバを初期化します。
     *
     * @return DataObjectBase HTTP リクエストを解析した結果のデータオブジェクト。
     */
    protected function& _parseHttpRequest()
    {
		// データオブジェクト取得
		$obj =& $this->getRequestObject();
		// HTTP リクエストを解析
		$obj->parseArray($_POST);
		//
		return $obj;
	}
	/**
	 * データフィールドの妥当性チェックを行います
	 *
	 * @param string[] $exclude チェックから除外するフィールドの配列。
	 * @return 異常があった場合は false を返します。
	 */
	protected function _validateField($exclude = array())
	{
		// ショートカット
		$VALID =& $this->_getValidator();
		// ルールを作成
		$rules = array();
		foreach($this->_validation_rules as $key => $value)
			$rules[$key] = $value[0];	// $value[0] ==> 検証ルール
		// ルールをセット
		$VALID->set_rules($rules);
		// 検証
		$res = $VALID->run();
		// エラーセット
		$this->_setValidateErrors();
		// 結果を復帰
		return $this->isValidate();
	}
	/**
	 * 直前の validation->run() のエラーメッセージを保存します。
	 * _validateField() の下請けです。
	 *
	 * @return void
	 */
	protected function _setValidateErrors()
	{
		// ショートカット
		$VALID =& $this->_getValidator();
		// エラー初期化
		$this->clearError();
		// エラーメッセージリストを作成
		foreach($this->_validation_rules as $key => $value)
		{
			$errorname = $key . '_error';
			if(!empty($VALID->$errorname))
			{
				$mes = @$value[1];	// ==> エラーメッセージ
				$this->setError($key, empty($mes)? $VALID->$errorname : $mes);
			}
		}
	}
	/**
	 * エラーメッセージを保存します
	 *
	 * @param string $name エラーの識別名
	 * @param string $descript エラーの内容を表す文字列
	 * @return void
	 */
	public function setError($name, $descript)
	{
		$this->_errors[$name] = $descript;
	}
	/**
	 * エラーメッセージをクリアします
	 *
	 * @return void
	 */
	public function clearError()
	{
		$this->_errors = array();
	}
	/**
	 * エラーメッセージの配列を返します
	 *
	 * @return string[] エラーメッセージの配列
	 */
	public function getErrors()
	{
		return $this->_errors;
	}
	/**
     * 書式化されたエラーメッセージを返します。
     *
     * @param string $seprow 複数のメッセージを区切る文字列
     * @return string エラーメッセージ
	 */
	public function getErrorMessage($seprow = "\n")
	{
		return join($seprow, $this->_errors);
	}
	/**
	 * データフィールドがすべて妥当ならば true を返します。
	 *
	 * @return 異常があった場合は false を返します。
	 */
	public function isValidate()
	{
		return (count($this->_errors) == 0);
	}
    /**
	 * データオブジェクトのインスタンスを返します。
	 * インスタンスが存在しなかった場合は新規に作成します。
	 *
	 * @return DataObjectBase $this->_object への参照
     */
    public function& getRequestObject()
    {
		if(is_null($this->_object))
			$this->_object = new $this->_object_name();
		return $this->_object;
	}
	/**
	 * CI の validation オブジェクトを返します
	 *
	 * @return CI_Validation オブジェクト
	 */
	protected function& _getValidator()
	{
		$this->load->library('validation');
		return $this->validation;
	}
}

/* End of file */