<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
require_once(APPPATH.'classes/ci_extend_libraries/ControllerBase.abstract.php');
require_once(APPPATH.'classes/dataobjects/TransactionResultData.class.php');

/**
 * PayProcessException
 */
class PayProcessException extends Exception
{
	const UNKNOWN				 		= 0;
	const FAIL_WRITE_REQUEST_LOG 		= 101;
	const FAIL_WRITE_SUBMIT_LOG 		= 102;
	const FAIL_WRITE_RESULT_LOG 		= 103;
	const FAIL_WRITE_RESPONSE_LOG 		= 104;
	const FAIL_WRITE_TRANSACTION_LOG 	= 105;
	const FAIL_WRITE_TRANSACTION_TRN 	= 106;

	/**
	 *
	 */
	public function __construct($code = self::UNKNOWN)
	{
		parent::__construct(null, $code);
	}
}

/**
 * PaymentControllerBase コントローラー抽象クラス
 */
abstract class PaymentControllerBase extends ControllerBase
{
	/**
	 * コンストラクタ
	 */
	public function __construct()
	{
		parent::__construct();
		// モデル先読み（微妙 orz）
		self::_getTableModel('TransactionLogModel');
		self::_getTableModel('TransactionTrnModel');
	}
	/**
	 * _execTransaction から呼び出されるコールバック。<br/>
	 * PaymentDriver の適切なメソッドをコールするよう実装します。<br/>
	 *
	 * @param PaymentDriver $paydrv PaymentDriver のインスタンス
	 * @param DataObject $obj_req リクエストデータオブジェクト
	 * @return bool 成功時は true
	 */
	abstract protected function _onSubmit(&$paydrv, &$obj_req);
	/**
	 * 使用する HttpRequestParserModel の名前を返すよう実装します。
	 *
	 * @return string 'models/transaction/' 配下のモデル名
	 */
	abstract protected function _getHttpRequestParserModelName();

	/**
	 * 決済システムへのトランザクションを実行する。<br/>
	 * 継承クラスの アクションからコールする。
	 *
	 * @return void
	 */
	protected function _execTransaction()
	{
		// リザルトデータ格納オブジェクト
		$obj_res = new TransactionResultData();
		$result = null;

		try
		{
			//
			$res = null;
			// Request パース
			$HTTPREQ =& $this->_createHttpRequestModel();
			$HTTPREQ->parseRequest();
			$obj_req = $HTTPREQ->getRequestObject();

			do
			{
				// Request ログ記録
				$body = $this->input->server('REQUEST_URI', true) . '?' . $obj_req->toNamevalueString();
				$trnlogid = $this->_writeRequestLog($body, $this->input->ip_address());
				// Validate
				if(!$HTTPREQ->isValidate())
				{
					$obj_res->setResultCode(current($HTTPREQ->getErrors()));
					break;
				}

				// PaymentDispatcherモデル初期化 ＆PaymentDriver 呼び出し
				$DISPATCHER =& $this->_getPaymentDispatcherModel();
				$res = $DISPATCHER->loginMerchant($obj_req->MERCHANTID, $obj_req->MERCHANTPW);
				if(!$res)
				{
					$obj_res->setResultCode(TransactionResultData::USER_AUTHENTICATION_FAILED);
					break;
				}
				$PAYDRV =& $DISPATCHER->getPaymentDriver();

				// ORIGID を検索
				$res = $this->_findPrevTransactionRequest($obj_req);
				if(!$res)
				{
					$obj_res->setResultCode(TransactionResultData::TRANSACTIONID_NOTFOUND);
					break;
				}
				$obj_req = $res;

				// Submit
				$res = $this->_onSubmit($PAYDRV, $obj_req);
				// Submit ログ記録
				$this->_writeSubmitLog($trnlogid, $PAYDRV->getSubmitValueString());

				if(!$res)
				{
					$obj_res->setResultCode(TransactionResultData::SUBMIT_EXECUTE_FAILED);
					break;
				}

				// Response ログ記録
				$this->_writeResponseLog($trnlogid, $PAYDRV->getResponseValueString());

				// Response → Result 変換
				$obj_res = $PAYDRV->convertResponseToResult();

				// 決済トランザクション記録 orz: もう一声！
				$this->_writeTransactionTrn(
					$PAYDRV->getTransactionId(),
					$PAYDRV->getOriginalTransactionCode(),
					$PAYDRV->getTransProcId(),
					$DISPATCHER->getMerchantInfo('id'),
					$obj_req->toNamevalueString());
			}
			while(false);

			// Result ログ記録
			$result = $obj_res->toNamevalueString();
			$this->_writeResultLog($trnlogid, $result);

		}
		catch(PayProcessException $e)
		{
			// 死すログ
			error_log($e);
		}
		catch(Exception $e)
		{
			// 死すログ
			error_log($e);
		}

		// Result
		if(is_null($result))
			$result = $obj_res->toNamevalueString();
		echo $result;
	}
	/**
	 * HttpRequestParserModel をロードして返します。_execTransaction() の下請け。
	 *
	 * @return HttpRequestParserModel httpリクエスト解析モデルのインスタンス
	 */
	protected function& _createHttpRequestModel()
	{
		$name = $this->_getHttpRequestParserModelName();
		$this->load->model('transaction/' . $name);
		$model =& $this->$name;
		return $model;
	}
	/**
	 * http リクエストを解析して httpリクエストデータオブジェクトを返します。_execTransaction() の下請け。
	 *
	 * @param HttpRequestParserModel $model_httpreq httpリクエスト解析モデルのインスタンス
	 * @return DataObjectBase httpリクエストデータオブジェクト
	 */
	protected function _getParseHttpRequest(&$model_httpreq)
	{
		// パース＆値検証
		$model_httpreq->parseRequest();
		// 値オブジェクトを取得
		return $model_httpreq->getRequestObject();
	}
	/**
	 * 送受信ログのリレーションを格納する TransactionLog  テーブルの更新。
	 *
	 * @param enum $logenum 更新するログ番号
	 * @param int $logid 更新するログのレコードID
	 * @param int $trnlogid トランザクションログのID
	 * @return int トランザクションログのID。更新失敗時は Exception
	 */
    protected function _writeTransactionLog($logenum, $logid, $trnlogid)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('TransactionLogModel');
		$res = $MODEL->writeLogId($logenum, $logid, $trnlogid);
		if(empty($res))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_TRANSACTION_LOG);
		// 復帰
		return $res;
    }
	/**
	 * ＧＷシステムの受信値を格納する RequestLog テーブルの更新。
	 *
	 * @param string $body ＧＷシステムの受信値の NameValue 文字列
	 * @param string $address 送信元のIPアドレス
	 * @return int トランザクションログのID。更新失敗時は Exception
	 */
    protected function _writeRequestLog($body, $address)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('RequestLogModel');
		$logid = $MODEL->writeLog($body, $address);
		if(empty($logid))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_REQUEST_LOG);
		// リンクログ書き込み
		$res = $this->_writeTransactionLog(TransactionLogModel::LOG_REQUEST, $logid, null);
		// 復帰
		return $res;
    }
	/**
	 * 決済システムへの送信値を格納する SubmitLog テーブルの更新。
	 *
	 * @param int $trnlogid トランザクションログのID
	 * @param string $body 決済システムへ送信する NameValue 文字列
	 * @return int トランザクションログのID。更新失敗時は Exception
	 */
    protected function _writeSubmitLog($trnlogid, $body)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('SubmitLogModel');
		$logid = $MODEL->writeLog($body);
		if(empty($logid))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_SUBMIT_LOG);
		// リンクログ書き込み
		$res = $this->_writeTransactionLog(TransactionLogModel::LOG_SUBMIT, $logid, $trnlogid);
		// 復帰
		return $res;
    }
	/**
	 * 決済システムからの戻り値を格納する ResponseLog テーブルの更新。
	 *
	 * @param int $trnlogid トランザクションログのID
	 * @param string $body 決済システムからの戻り値の文字列
	 * @return int トランザクションログのID。更新失敗時は Exception
	 */
    protected function _writeResponseLog($trnlogid, $body)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('ResponseLogModel');
		$logid = $MODEL->writeLog($body);
		if(empty($logid))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_RESPONSE_LOG);
		// リンクログ書き込み
		$res = $this->_writeTransactionLog(TransactionLogModel::LOG_RESPONSE, $logid, $trnlogid);
		// 復帰
		return $res;
    }
	/**
	 * ＧＷシステムからの戻り値を格納する ResultLog テーブルの更新。
	 *
	 * @param int $trnlogid トランザクションログのID
	 * @param string $body ＧＷシステムからの戻り値の NameValue 文字列
	 * @return int トランザクションログのID。更新失敗時は Exception
	 */
    protected function _writeResultLog($trnlogid, $body)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('ResultLogModel');
		$logid = $MODEL->writeLog($body);
		if(empty($logid))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_RESULT_LOG);
		// リンクログ書き込み
		$res = $this->_writeTransactionLog(TransactionLogModel::LOG_RESULT, $logid, $trnlogid);
		// 復帰
		return $res;
    }
	/**
	 * 決済トランザクションテーブル TransactionTrn の更新。
	 *
	 * @param string $trnid ＧＷシステムのトランザクションID
	 * @param string $orgtrnid 決済システムのトランザクションID
	 * @param enum $processed 実行済みのトランザクションのステップ
	 * @param string $merchantid MerchantTbl のレコードID
	 * @param string $request リクエストデータ Namevalue 文字列
	 * @return boolean ture。更新失敗時は Exception
	 */
    protected function _writeTransactionTrn($trnid, $orgtrnid, $processed, $merchantid, $request)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('TransactionTrnModel');
		$res = $MODEL->writeValue($trnid, $orgtrnid, $processed, $merchantid, $request);
		if(empty($res))
			throw new PayProcessException(PayProcessException::FAIL_WRITE_RESULT_LOG);
		// 復帰
		return $res;
    }
	/**
     * 決済システムのトランザクションIDから TransactionTrn テーブルを検索し、
     * 該当すればそのリクエストオブジェクトを返す。
     *
     * @param DataObject $obj_req 今回送信されてきたリクエストオブジェクト
     * @return DataObject 該当の決済システムのトランザクションIDがあればその時のリクエストオブジェクト。<br/> なければ今回のリ
     * クエストオブジェクト
	 */
	protected function _findPrevTransactionRequest($obj_req)
	{
		// 含まれていない場合は復帰
		if(!isset($obj_req->TRANSID))
			return $obj_req;
		// 検索
		$row_trans = $this->_getTransactionTrn($obj_req->TRANSID);
		if(!$row_trans)
			return null;
//			return $obj_req;
		// リクエストオブジェクトにマージ
		$obj_req->parseNamevalueString($row_trans['orig_request_values']);
		$obj_req->ORIGID = $row_trans['orig_trn_id'];
		//
		return $obj_req;
	}
	/**
     * ＧＷシステムのトランザクションIDから TransactionTrn テーブルの情報を返す。
     *
     * @param string $trnid ＧＷシステムのトランザクションID
     * @return string 指定IDの TransactionTrn 情報
	 */
    protected function _getTransactionTrn($trnid)
    {
		// ログ書き込み
		$MODEL =& self::_getTableModel('TransactionTrnModel');
		$res = $MODEL->getValue($trnid);
		// 復帰
		return $res;
    }
	/**
     * PaymentDriver ディスパッチャーモデル取得
	 *
	 * @return datatype description
	 */
	protected function& _getPaymentDispatcherModel()
	{
		// Payment モデル呼び出し
		$this->load->model('payment/paymentdispatcher');
		return $this->paymentdispatcher;
	}
    /**
     * テーブルアクセス用モデル取得
     *
     * @param string $name テーブルアクセス用モデル名
     * @return Model テーブルアクセス用モデルのインスタンス
     */
    protected static function& _getTableModel($name)
    {
		$CI =& get_instance();
		$CI->load->model('database/tables/' . $name, '', true);
		return $CI->$name;
    }
    /**
     * TODO2: エラーメッセージ検索
     */
}

/* End of file */