<?php
require_once('DataObjectBase.class.php');

/**
 * TransactionResponseData クラス
 */
class TransactionResponseData extends DataObjectBase
{
	/**
	 * 決済システムからの生レスポンスデータ
	 */
	protected $_raw_response = null;
	/**
	 * http リクエストオブジェクトを解析してメンバーをセットします。
	 * デフォルトは単純コピーです。
	 *
	 * @param mixed $response 決済システムからのレスポンスデータ
	 * @param mixed $options 継承クラス用追加オプション
	 * @return void
	 */
	public function parseResponseValue($response, $options = null)
	{
		$this->_raw_response = $response;
	}
	/**
	 * 決済システムからの生レスポンスデータを返します
	 *
	 * @return mixed 決済システムからの生レスポンスデータ
	 */
	public function getRawResponse()
	{
		return $this->_raw_response;
	}
	/**
	 * 決済システムのオリジナルの結果コードを返します。
	 *
	 * @abstract
	 * @return string オリジナルの結果コード
	 */
	public function getResultCode()
	{
		return '';
	}
	/**
	 * 決済システムのオリジナルの結果メッセージを返します。
	 *
	 * @abstract
	 * @return string オリジナルの結果メッセージ
	 */
	public function getResultMessage()
	{
		return '';
	}
	/**
	 * 決済システムのオリジナルのトランザクションコードを返します。
	 *
	 * @abstract
	 * @return string オリジナルのトランザクションコード
	 */
	public function getTransactionCode()
	{
		return '';
	}
	/**
	 * 結果が成功かどうか返します。
	 *
	 * @abstract
	 * @return boolean 成功なら true
	 */
	public function isSucceed()
	{
		return true;
	}
	/**
	 * Void が可能かどうか返します。
	 *
	 * @abstract
	 * @return boolean 可能なら true
	 */
    public function isAvailableVoid()
    {
    	return false;
    }
}

/* End of file */