<?php
require_once(APPPATH.'classes/dataobjects/TransactionResponseData.class.php');
require_once(APPPATH.'classes/dataobjects/TransactionResultData.class.php');

/**
 * MtrexResponseData クラス
 */
class MtrexResponseData extends TransactionResponseData
{
	/**
	 * 既定メンバーの名前のリスト
	 */
	/*
		mp_APIVersionDiscontinuanceDate=
		mp_AuthCode=T23456
		mp_AVSAddressResult=Unavailable
		mp_AVSCombinedResult=Unavailable
		mp_AVSPostalCodeResult=Unavailable
		mp_AVSProcessorResultCode=U
		mp_CardValidationProcessorResultCode=P
		mp_CardValidationResult=NotProcessed
		mp_CustomTransID=cbff01539dc60f59559c59502d5030c8
		mp_IsTestTrans=True
		mp_IsTransUpdateable=True
		mp_ResponseInstanceCode=F44B0F13-2D29-45B5-8350-38029E1C8BE2
		mp_TransID=69110
		mp_TransProcessorResultCode=TEST
		mp_TransResult=Approved
		mp_TransResultMessage=
		mp_TransResultReasonCode=0
	 */
	protected $_propertynames = array(
		'mp_APIVersionDiscontinuanceDate',
		'mp_AuthCode',
		'mp_AVSAddressResult',
		'mp_AVSCombinedResult',
		'mp_AVSPostalCodeResult',
		'mp_AVSProcessorResultCode',
		'mp_CardValidationProcessorResultCode',
		'mp_CardValidationResult',
		'mp_CustomTransID',
		'mp_IsTestTrans',
		'mp_IsTransUpdateable',
		'mp_ResponseInstanceCode',
		'mp_TransID',
		'mp_TransProcessorResultCode',
		'mp_TransResult',
		'mp_TransResultMessage',
		'mp_TransResultReasonCode',
	);
	/**
	 * コンストラクタ
	 */
	public function __construct($datas = array())
	{
		parent::__construct($datas);
	}
	/**
	 * http リクエストオブジェクトを解析してメンバーをセットします。
	 * デフォルトは単純コピーです。
	 *
	 * @param mixed $response 決済システムからのレスポンスデータ
	 * @param mixed $options 継承クラス用追加オプション
	 * @return void
	 */
	public function parseResponseValue($response, $options = null)
	{
		// 親
		$this->_raw_response = $response;
		// レスポンス文字列からレスポンスオブジェクトを初期化
		$this->parseNamevalueString($response);
	}
	/**
	 * 決済システムのオリジナルの結果コードを返します。
	 *
	 * @abstract
	 * @return string オリジナルの結果コード
	 */
	public function getResultCode()
	{
		return $this->mp_TransResultReasonCode;
	}
	/**
	 * 決済システムのオリジナルの結果メッセージを返します。
	 *
	 * @abstract
	 * @return string オリジナルの結果メッセージ
	 */
	public function getResultMessage()
	{
		$mes = array();
		$mes[] = $this->mp_TransResult;
		$mes[] = $this->mp_TransResultMessage;
		return join("\r\n", array_filter($mes));
	}
	/**
	 * 決済システムのオリジナルのトランザクションコードを返します。
	 *
	 * @abstract
	 * @return string オリジナルのトランザクションコード
	 */
	public function getTransactionCode()
	{
		return $this->mp_TransID;
	}
	/**
	 * 結果が成功かどうか返します。
	 *
	 * @abstract
	 * @return boolean 成功なら true
	 */
	public function isSucceed()
	{
		return $this->mp_TransResultReasonCode == 0;
	}
	/**
	 * Void が可能かどうか返します。
	 *
	 * @abstract
	 * @return boolean 可能なら true
	 */
    public function isAvailableVoid()
    {
    	return (strcasecmp($this->mp_IsTransUpdateable, 'True') == 0);
    }
}

/* End of file */