<?php
require_once(APPPATH.'classes/dataobjects/TransactionSubmitData.class.php');

/**
 * MtrexSubmitData クラス
 */
class MtrexSubmitData extends TransactionSubmitData
{
	/**
	 * 既定メンバーの名前のリスト
	 * 決済システムパラメータ => ＧＷシステムパラメータ
	 */
	protected $_propertynames = array(
		//
		'mp_APIVersion',
		'mp_AuthenticationID',
		'mp_ConfigID',
		'mp_AuthenticationPassword',
		'mp_MessageRequestType',
		'mp_TransClearingAction',
		//
		'mp_DuplicateBehaviorType',
		'mp_PaymentMethod',
		'mp_IsRecurringTrans',
		//
		'mp_IsTestTrans',
		'mp_TestInstructions',
		//
		'mp_NewCustomTransID',
		'ORIGID' => 'mp_TransID',
		'TRANSID' => 'mp_CustomTransID',
		'mp_TransIDRef',
		'mp_CustomTransIDRef',
		//
		'mp_OrderNumber',
		'mp_PurchaseOrderNumber',
		//
		'AMOUNT' => 'mp_TotalAmount',
		'mp_TotalTaxAmount',
		'CURRENCY' => 'mp_CurrencyCode',
		'NAME' => 'mp_NameOnCard',
		'CRDNUM' => 'mp_CardNumber',
		'CRDEXP' => 'mp_CardExpirationDate',
		'CVV2' => 'mp_CardValidationValue',
		//
		'mp_IsTransAlreadyAuthed',
		'mp_AuthCode',
		//
		'mp_ABARoutingNumber',
		'mp_BankAccountNumber',
		'mp_BankAccountType',
		//
		'FIRSTNAME' => 'mp_BillingFirstName',
		'LASTNAME' => 'mp_BillingLastName',
		'COMPANY' => 'mp_BillingCompany',
		'STREET' => 'mp_BillingAddressLine1',
		'mp_BillingAddressLine2',
		'mp_BillingAddressLine3',
		'CITY' => 'mp_BillingCity',
		'STATE' => 'mp_BillingState',
		'COUNTRY' => 'mp_BillingCountryCode',
		'ZIP' => 'mp_BillingPostalCode',
		'PHONE' => 'mp_BillingPhone',
		//
		'mp_ShippingFirstName',
		'mp_ShippingLastName',
		'mp_ShippingCompany',
		'mp_ShippingAddressLine1',
		'mp_ShippingAddressLine2',
		'mp_ShippingAddressLine3',
		'mp_ShippingCity',
		'mp_ShippingState',
		'mp_ShippingCountryCode',
		'mp_ShippingPostalCode',
		'mp_ShippingPhone',
		//
		'mp_CustomerIPAddress',
		'mp_MerchantEmailToAddress',
		//
		'COMMENT1' => 'Comment1',
		'COMMENT2' => 'Comment2',
	);
	/**
	 * コンストラクタ
	 */
	public function __construct($datas = array())
	{
		parent::__construct($datas);
	}
	/**
	 * http リクエストオブジェクトを解析してメンバーをセットします。
	 *
	 * @param TransactionRequestData $obj http リクエストオブジェクト
	 * @param mixed $options 継承クラス用追加オプション
	 * @return void
	 */
	public function parseRequestObject($request, $options = null)
	{
		//
		$defopt = array(
			'MessageRequestType'	=> null,
			'TransClearingAction'	=> null,
			'PaymentInfo'			=> null,
			'TransID'				=> null,
		);
		$options = array_merge($defopt, $options);
		// パラメータをサブミットオブジェクトにセット
		$this->parseArray($request->toArray(), $this->_propertynames);
		// 専用パラメータ設定
		$this->mp_APIVersion	 	= '1.6';			// 固定
		$this->mp_PaymentMethod		= 'CreditCard';		// 固定
		// アカウント情報
		$payinfo =& $options['PaymentInfo'];
		$this->mp_AuthenticationID			= $payinfo->account;
		$this->mp_AuthenticationPassword	= $payinfo->password;
		$this->mp_ConfigID					= $payinfo->account_ex1;
		$this->mp_IsTestTrans				= ($payinfo->account_ex2 == 1)? true : false;
		$this->mp_MessageRequestType		= $options['MessageRequestType'];
		$this->mp_TransClearingAction		= $options['TransClearingAction'];
		if(empty($request->TRANSID))
			$obj_smt->mp_NewCustomTransID	= $options['TransID'];
		else
			$obj_smt->mp_CustomTransID		= $options['TransID'];
		//
		// orz
		switch($options['MessageRequestType'])
		{
			case 'Inquiry':
				$copy = $this->toArray(self::_getInquiryFieldNames());
				break;
			case 'Refund':
				$copy = $this->toArray(self::_getRefundFieldNames());
				break;
			case 'Update':
				$copy = $this->toArray(self::_getUpdateFieldNames());
				break;
			default:
				$copy = $this->toArray();
				break;
		}
		$this->emptyObject();
		$this->parseArray($copy);
	}
	/**
	 *
	 */
 	public function toString_mp_CardExpirationDate()
	{
		$value = $this->mp_CardExpirationDate;
		// からならそのまま
		if(empty($value))
			return $value;
		// 数値６桁ならそのまま返す
		if(preg_match('/[01]\d20\d{2}/', $value))
			return $value;
		//
		$res = substr($value, 0, 2) . '20' . substr($value, 2, 2);
		return $res;
	}
	/**
	 *
	 */
	public static function _getRefundFieldNames()
	{
		$values = array(
			//
			'mp_APIVersion',
			'mp_AuthenticationID',
			'mp_ConfigID',
			'mp_AuthenticationPassword',
			'mp_MessageRequestType',
			'mp_TransClearingAction',
			'mp_DuplicateBehaviorType',
			//
			'mp_IsTestTrans',
			'mp_TestInstructions',
			//
//			'mp_TransID',
			'mp_CustomTransID',
			//
			'mp_OrderNumber',
			'mp_PurchaseOrderNumber',
			//
			'mp_TotalAmount',
			'mp_TotalTaxAmount',
			'mp_CurrencyCode',
			'mp_CardExpirationDate',
		);
		return $values;
	}
	/**
	 *
	 */
	public static function _getUpdateFieldNames()
	{
		$values = array(
			//
			'mp_APIVersion',
			'mp_AuthenticationID',
			'mp_ConfigID',
			'mp_AuthenticationPassword',
			'mp_MessageRequestType',
			'mp_TransClearingAction',
			//
			'mp_IsTestTrans',
			'mp_TestInstructions',
			//
//			'mp_TransID',
			'mp_CustomTransID',
			//
			'mp_TotalAmount',
			'mp_TotalTaxAmount',
			'mp_CurrencyCode',
		);
		return $values;
	}
	/**
	 *
	 */
	public static function _getInquiryFieldNames()
	{
		$values = array(
			//
			'mp_APIVersion',
			'mp_AuthenticationID',
			'mp_ConfigID',
			'mp_AuthenticationPassword',
			'mp_MessageRequestType',
			//
			'mp_IsTestTrans',
			'mp_TestInstructions',
			//
//			'mp_TransID',
			'mp_CustomTransID',
			'mp_TransIDRef',
			'mp_CustomTransIDRef',
		);
		return $values;
	}
}

/* End of file */