<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once(APPPATH.'classes/ci_extend_libraries/PaymentDriverBase.abstract.php');
require_once('MtrexSubmitData.class.php');
require_once('MtrexResponseData.class.php');

/**
 * Mtrex.driver クラス
 */
class MtrexPaymentDriver extends PaymentDriverBase
{
	/**
	 * @property string $_submit_object_name サブミットオブジェクトのクラス名
	 */
	protected $_submit_object_name = 'MtrexSubmitData';
	/**
	 * @property string $_response_object_name レスポンスオブジェクトのクラス名
	 */
	protected $_response_object_name = 'MtrexResponseData';
	/**
	 * コンストラクタ
	 */
    public function __construct()
    {
        parent::__construct();
    }

	/**
	 * sale transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @param string $transid 確認するトランザクションコード。オプション
	 * @return boolean サブミットできれば true。
	 */
	public function transactionSale($paramobj)
	{
		$this->_last_trans_proc_id = ConstStatus::TRN_SALE;
		return $this->_submitPayment('Sale', 'Capture', $paramobj);
	}
   	/**
     * authorization transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @return boolean サブミットできれば true。
     */
    public function transactionAuthorization($paramobj)
	{
		$this->_last_trans_proc_id = ConstStatus::TRN_AUTHORIZATION;
		return $this->_submitPayment('Sale', 'Hold', $paramobj);
	}
    /**
     * delayed capture transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @return boolean サブミットできれば true。
     */
    public function transactionDelayedCapture($paramobj)
	{
		$this->_last_trans_proc_id = ConstStatus::TRN_DELAYEDCAPTURE;
		return $this->_submitPayment('Update', 'Capture', $paramobj);
	}
	/**
	 * void transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @return boolean サブミットできれば true。
	 */
    public function transactionVoid($paramobj)
	{
		$this->_last_trans_proc_id = ConstStatus::TRN_VOID;
		return $this->_submitPayment('Update', 'Void', $paramobj);
	}
	/**
	 * credit transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @return boolean サブミットできれば true。
	 */
    public function transactionCredit($paramobj)
	{
		$this->_last_trans_proc_id = ConstStatus::TRN_CREDIT;
		return $this->_submitPayment('Refund', 'Capture', $paramobj);
	}

	/**
     * inquiry transaction 実実行
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @return boolean サブミットできれば true。
	 */
    protected function _transactionInquiry($paramobj)
	{
		return $this->_submitPayment('Inquiry', null, $paramobj);
	}

	/**
	 * Void が可能かどうか確認します
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @return boolean void が可能なら true。
	 */
    public function isAvailableVoid($paramobj)
	{
		$this->_transactionInquiry($paramobj);
		$obj_res =& $this->getResponseObject();
		return $obj_res->isAvailableVoid();
	}

    /**
     * サブミットを実行します
     * @param string $msgtype MessageRequestType
     * @param string $action TransClearingAction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @return boolean サブミットが成功すれば true
     */
    protected function _submitPayment($msgtype, $action, $paramobj)
    {
    	$payinfo =& $this->_payment_info;

		// トランザクションIDをインスタンスに設定（新規の場合は新IDを取得）
		$transid = $paramobj->TRANSID;
		$newtransid = $this->setTransactionId($transid);

		// パラメータをサブミットオブジェクトにセット
		$obj_smt =& $this->getSubmitObject();
		$options = array(
			'MessageRequestType'	=> $msgtype,
			'TransClearingAction'	=> $action,
			'PaymentInfo'			=> $payinfo,
			'TransID'				=> $newtransid,
		);
		$obj_smt->parseRequestObject($paramobj, $options);
		// サブミット用オブジェクトからパラメータ文字列の取得
		$namevalue = $obj_smt->toNamevalueString();

		// サブミット実行
//var_dump($namevalue);
		$response  = $this->_execCurl($namevalue, $newtransid, $payinfo->submit_url);
//$response = 'RESULT=0&RESPMSG=GEMA&PNREF=1234567890';
//var_dump($response);

		// レスポンス用オブジェクトを設定
		if($response === false)
			return false;
		else
		{
			$obj_rps =& $this->getResponseObject();
			$obj_rps->parseResponseValue($response);
			return true;
		}
    }
    /**
     * リクエストヘッダを作成して返します
	 * @param string $transid ＧＷシステムのトランザクションコード
	 * @return atring[] httpヘッダ文字列の配列
     */
    protected function _makeCurlHeaders($transid)
    {
    	//
		$curl_headers = array(
//			'Content-Type: text/namevalue',
//			'X-VPS-Timeout: 30',
//			'X-VPS-VIT-OS-Name: Linux',
//			'X-VPS-VIT-OS-Version: RHEL 5.2',
//			'X-VPS-VIT-Client-Type: PHP/cURL',
//			'X-VPS-VIT-Client-Version: 0.01',
//			'X-VPS-VIT-Client-Architecture: x86',
//			'X-VPS-VIT-Integration-Product: PaymentGW',
//			'X-VPS-VIT-Integration-Version: 0.01',
		);
//    	$curl_headers[] = "X-VPS-Request-ID: $transid";
    	//
    	return $curl_headers;
    }
    /**
     * リクエストを発行します
     *
     * @param string $namevalue 送信するパラメータリスト
     * @param string $transid 内部生成したトランザクションID
     * @param string $submit_url 送信する URL
     * @return string 発行先から戻されたレスポンス文字列
     */
    protected function _execCurl($namevalue, $transid, $submit_url)
    {
    	//
		$ch = curl_init();
		//
//		$submit_url = $this->_payment_info->submit_url;
//		$transid = $this->getTransactionId();
		$headers = $this->_makeCurlHeaders($transid);
		//
		curl_setopt($ch, CURLOPT_URL, $submit_url);
//		curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
//		curl_setopt($ch, CURLOPT_USERAGENT, $user_agent);
//		curl_setopt($ch, CURLOPT_HEADER, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 45);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $namevalue);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,  2);
		curl_setopt($ch, CURLOPT_FORBID_REUSE, true);
		curl_setopt($ch, CURLOPT_POST, 1);
		//
		$res = curl_exec($ch);
		//
		curl_close($ch);
		//
		return $res;
    }
}

/* End of file */