<?php
require_once(APPPATH.'classes/dataobjects/TransactionResponseData.class.php');
require_once(APPPATH.'classes/dataobjects/TransactionResultData.class.php');

/**
 * PaypalResponseData クラス
 */
class PaypalResponseData extends TransactionResponseData
{
	/**
	 * 既定メンバーの名前のリスト
	 */
	protected $_propertynames = array(
		// main
		'RESULT'	=> 'RESULT',
		'PNREF'		=> 'PNREF',
		'PPREF'		=> 'PPREF',
		'RESPMSG'	=> 'RESPMSG',
		'AUTHCODE'	=> 'AUTHCODE',
		// extra
		'CVV2MATCH'	=> 'CVV2MATCH',
		'AVSADDR'	=> 'AVSADDR',
		'AVSZIP'	=> 'AVSZIP',
		'IAVS'		=> 'IAVS',
		'PROCAVS'	=> 'PROCAVS',
		'PROCCVV2'	=> 'PROCCVV2',
		'PAYMENTTYPE'	=> 'PAYMENTTYPE',
		'CORRELATIONID'	=> 'CORRELATIONID',
		'BALAMT'		=> 'BALAMT',
		'AMEXID'		=> 'AMEXID',
		'AMEXPOSDATA'	=> 'AMEXPOSDATA',
		'TRANSSTATE'	=> 'TRANSSTATE',
	);
	/**
	 * コンストラクタ
	 */
	public function __construct($datas = array())
	{
		parent::__construct($datas);
	}
	/**
	 * http リクエストオブジェクトを解析してメンバーをセットします。
	 * デフォルトは単純コピーです。
	 *
	 * @param mixed $response 決済システムからのレスポンスデータ
	 * @param mixed $options 継承クラス用追加オプション
	 * @return void
	 */
	public function parseResponseValue($response, $options = null)
	{
		// 親
		$this->_raw_response = $response;
		// レスポンス文字列からレスポンスオブジェクトを初期化
		$resvalues = strstr($response, 'RESULT');
		if($resvalues)
			$this->parseNamevalueString($resvalues);
	}
	/**
	 * 決済システムのオリジナルの結果コードを返します。
	 *
	 * @return string オリジナルの結果コード
	 */
	public function getResultCode()
	{
		$res = null;
		// TODO2: おひっこし
		if($this->RESULT == 0)
			$res = TransactionResultData::APPROVED;
		elseif($this->RESULT > 0)
			$res = $this->RESULT + 10000;
		elseif($this->RESULT < 0)
			$res = abs($this->RESULT) + 20000;
		//
		return $res;
	}
	/**
	 * 決済システムのオリジナルの結果メッセージを返します。
	 *
	 * @return string オリジナルの結果メッセージ
	 */
	public function getResultMessage()
	{
		return $this->RESPMSG;
	}
	/**
	 * 決済システムのオリジナルのトランザクションコードを返します。
	 *
	 * @return string オリジナルのトランザクションコード
	 */
	public function getTransactionCode()
	{
		return $this->PNREF;
	}
	/**
	 * 結果が成功かどうか返します。
	 *
	 * @return boolean 成功なら true
	 */
	public function isSucceed()
	{
		return $this->RESULT == 0;
	}
	/**
	 * Void が可能かどうか返します。
	 *
	 * @return boolean 可能なら true
	 */
	public function isAvailableVoid()
	{
		//
		$res = false;
		//
		switch($this->TRANSSTATE)
		{
			// PayflowPro_Guide P.162 参照
			case 3:
			case 6:
			case 7:
				$res = true;
				break;
			//
			default:
				$res = false;
				break;
		}
		//
		return $res;
	}
}

/* End of file */