<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once(BASEPATH.'libraries/Model.php');
/**
 * PaymentDriver ディスパッチャーモデル
 */
class PaymentDispatcher extends Model
{
	/**
	 * MerchantTbl 情報インスタンス
	 * @property MerchantTbl $_merchant_info MerchantTbl のインスタンス
	 */
	protected $_merchant_info = null;

	/**
	 * コンストラクタ
	 */
    public function __construct()
    {
        parent::__construct();
    }
    /**
     * ＧＷシステムのログインＩＤ／ＰＷから決済システムのアクセス情報を取得する。
     * 取得できた場合、以下の情報が初期化される。
     * <ul>
     * <li>$this->_merchant_info に MerchantTbl の取得レコード情報がセットされる</li>
     * <li>MerchantTbl にリンクした PaymentGwMst のレコードを元に PaymentDriver がロードされる</li>
     * <li>ロードされた PaymentDriver に PaymentGwMst の情報がセットされる</li>
     * </ul>
     *
     * @param string $loginid ＧＷシステムのログインＩＤ
     * @param string $passwd ＧＷシステムのパスワード
     * @return boolean アクセス情報を取得できれば true
     */
    public function loginMerchant($loginid, $passwd)
    {
    	// MerchantTbl 読み込み
		$MODEL =& self::_getTableModel('MerchantTblModel');
		$res_mer = $MODEL->getAccount($loginid, $passwd);
		if(empty($res_mer))
			return false;
		$this->_merchant_info = $res_mer;
    	// PaymentGwMst 読み込み
		$MODEL =& self::_getTableModel('PaymentGwMstModel');
		$res_pay = $MODEL->getPKey($res_mer->payment_gw_id);
		if(empty($res_pay))
			return false;
		// ドライバー読み込み
    	if(!$this->_loadPaymentDriver($res_pay->name))
    		return false;
    	// ドライバー初期化
    	$DRIVER =& $this->getPaymentDriver();
//		$DRIVER->initialize($res_mer, $res_pay);
		$DRIVER->initialize($res_pay);
    	//
    	return true;
    }
    /**
     * PaymentDriver のインスタンスを返します。
     *
     * @return PaymentDriver PaymentDriver のインスタンス
     */
    public function& getPaymentDriver()
    {
		$CI =& get_instance();
		return $CI->PaymentDriver;
    }
    /**
     * 取得済みの MerchantTbl の情報を返す。
     *
     * @param string $member 得たい MerchantTbl のフィールド名。null 指定時は全てのフィールド
     * @return string|array MerchantTbl のフィールドの値。または MerchantTbl オブジェクト
     */
    public function getMerchantInfo($member = null)
    {
    	if(empty($member))
			return $this->_merchant_info;
		else
			return $this->_merchant_info->$member;
    }
    /**
     * PaymentDriver をロードする。成功時は 'PaymentDriver' というモデル名で利用できるようになる。
     *
     * @param string $driver ロードする PaymentDriver モデル名。
     * @return bool ロードできれば true
     */
    protected function _loadPaymentDriver($driver)
    {
		$CI =& get_instance();
		$path = "payment/drivers/${driver}/${driver}PaymentDriver";
		$CI->load->model($path, 'PaymentDriver', true);
		//
		return !empty($CI->PaymentDriver);
    }
    /**
     * テーブルアクセス用モデル取得
     *
     * @param string $name テーブルアクセス用モデル名
     * @return Model テーブルアクセス用モデルのインスタンス
     */
    protected static function& _getTableModel($name)
    {
		$CI =& get_instance();
		$CI->load->model('database/tables/' . $name, '', true);
		return $CI->$name;
    }
}

/* End of file */