<?php

/**
 * ProxyBase クラス
 */
class NamevalueProxyBase
{
	/**
	 * サブミット URL
	 * @access protected
	 */
	var $_submit_url = null;
	/**
	 * user_agent
	 * @access protected
	 */
	var $_user_agent = null;
	/**
	 * コンストラクタ
     * @access public
	 */
    function NamevalueProxyBase()
    {
    }

	/**
	 * execSubmit
	 * シングルトンの自分のインスタンスを作成し、submit メソッドを実行します。
     * このメソッドは、必ず各サブクラスで同様の実装でオーバーライドします。
     * @abstract
	 * @access public
	 * @static
	 * @param mixed $options 追加オプション
	 * @return void
	 */
	function execSubmit($options = null)
	{
		// インスタンス取得
		$proxy =& ProxyBase::_getInstance(__CLASS__);
//		$proxy =& parent::_getInstance(__CLASS__);
		// submit 実行
		$proxy->submit();
	}
    /**
     * getInstance
     * シングルトンの自分のインスタンスを返します。
     * このメソッドは、必ず各サブクラスで同様の実装でオーバーライドします。
     * @abstract
     * @access public
     * @static
     * @return object 自分自身のインスタンス
     */
//	function &getInstance()
//	{
//		return ProxyBase::_getInstance(__CLASS__);
//	}

    /**
     * submit
     * サブミットを実行します。
	 * @param mixed $options 追加オプション
     * @access public
     */
    function submit($options = null)
    {
    	//
    	$result = null;
    	//
    	do
    	{
	    	// 初期化
	    	if(!$this->_submitInitialize($options))
	    		break;
	    	// ヘッダー処理
	    	$header = $this->_submitHeader();
	    	// リクエスト処理
	    	$request = $this->_submitRequest();
	    	if(empty($request))
	    		break;
	    	// サブミット
	    	$result = $this->_submitExec($header, $request);
    	}
    	while(false);

	    // レスポンス
		$this->_submitResponse($result);
    }
    /**
     * _submitInitialize
     * コールバック。サブミット時の初期化処理を行います。
     * @access protected
	 * @param mixed $options 追加オプション
     * @return bool submit を中断する場合は false
     */
    function _submitInitialize($options = null)
    {
    	return true;
    }
    /**
     * _submitHeader
     * コールバック。受け取ったヘッダーを解析します。ヘッダーを連想配列にして戻します。
     * @access protected
     * @return array ヘッダーの連想配列
     */
    function _submitHeader()
    {
    	return getallheaders();
    }
    /**
     * _submitRequest
     * コールバック。受け取ったリクエストを解析します。リクエストを連想配列にして戻します。
     * @access protected
     * @return array リクエストの連想配列
     */
    function _submitRequest()
    {
    	// $_POST 補完（Content-Type: text/namevalue だと $_POST が定義されない対策）
    	if(empty($_POST))
    	{
	    	$req = file_get_contents('php://input');
			mb_parse_str($req, $_POST);
    	}
    	//
    	return $_POST;
    }
    /**
     * _submitExec
     * コールバック。決済システムに実際のサブミットを行います。レスポンスをそのまま戻します。
     * @access protected
     * @param array $header ヘッダーの連想配列
     * @param array $request リクエストの連想配列
     * @return string レスポンス文字列
     */
    function _submitExec($header, $request)
    {
    	return null;
    }
    /**
     * _submitResponse
     * コールバック。決済システムのサブミットの結果をリクエスト側に返します。
     * @access protected
     */
    function _submitResponse($response)
    {
    	echo $response;
    }
    /**
     * _getInstance
     * getInstance から呼ばれます。シングルトンの自分のインスタンスを返します。
     * @access protected
     * @static
     * @param string $thisname 自分自身のクラスの名前
     * @return object 自分自身のインスタンス
     */
	function &_getInstance($thisname)
	{
		static $instance = array();

		if(!isset($instance[$thisname]))
			$instance[$thisname] =& new $thisname();
		return $instance[$thisname];
	}
}

/* End of file */