<?php

require_once('NamevalueProxyBase.class.php');

/**
 * ProxyBase クラス
 */
class PayPalProxy extends NamevalueProxyBase
{
	/**
	 * コンストラクタ
     * @access public
	 */
    function PayPalProxy()
    {
    	parent::NamevalueProxyBase();
    }
	/**
	 * execSubmit
	 * シングルトンの自分のインスタンスを作成し、submit メソッドを実行します。
     * このメソッドは、必ず各サブクラスで同様の実装でオーバーライドします。
     * @abstract
	 * @access public
	 * @static
	 * @param mixed $options 追加オプション
	 * @return void
	 */
	function execSubmit($options = null)
	{
		// インスタンス取得
		$proxy =& parent::_getInstance(__CLASS__);
		// submit 実行
		$proxy->submit();
	}
    /**
     * _submitInitialize
     * コールバック。サブミット時の初期化処理を行います。
     * @access protected
	 * @param mixed $options 追加オプション
     * @return bool submit を中断する場合は false
     */
    function _submitInitialize($options = null)
    {
		// デフォルトオプション
		$defpot = array(
			'Live' => false,
			'LiveUrl' => 'https://payflowpro.paypal.com',
			'TestUrl' => 'https://pilot-payflowpro.paypal.com',
			'UserAgent' => 'PHP/' . phpversion(),
		);
		// オプションマージ
		$options = array_merge($defpot, (array)$options);
		// submit url 設定
		$this->_submit_url = $options['Live']? $options['LiveUrl'] : $options['TestUrl'];
		// その他設定
		$this->_user_agent = $options['UserAgent'];
		// 復帰
    	return !empty($this->_submit_url);
    }
    /**
     * _submitHeader
     * コールバック。受け取ったヘッダーを解析します。ヘッダーを連想配列にして戻します。
     * @access protected
     * @return array ヘッダーの配列
     */
    function _submitHeader()
    {
    	// デフォルト
    	$default = array(
    		'Content-Type'	=> 'text/namevalue',
    	);
    	// 元ヘッダ
    	$headers = parent::_submitHeader();
    	$headers = array_merge($default, $headers);
    	//
    	$curl_headers = array();
    	// デフォルトヘッダ
    	foreach(array_keys($default) as $key)
    	{
			$value = $headers[$key];
			$curl_headers[] = "{$key}: {$value}";
    	}
    	// 追加ヘッダ
    	foreach($headers as $key => $value)
    		if(strpos($key, 'X-VPS') === 0)
    			$curl_headers[] = "{$key}: {$value}";
    	// 戻り
    	return $curl_headers;
    }
    /**
     * _submitExec
     * コールバック。決済システムに実際のサブミットを行います。レスポンスをそのまま戻します。
     * @access protected
     * @param array $header ヘッダーの配列
     * @param array $request リクエストの連想配列
     * @return string レスポンス文字列
     */
    function _submitExec($header, $request)
    {
//    	$response = "RESULT=999&RESPMSG=GEMA&PNREF=1234567890";
    	$response = $this->__execCurl($header, $request);
    	//
    	return $response;
    }

    /**
     * リクエストを発行します
     *
     * @access private
     * @param array $header ヘッダーの配列
     * @param array $request リクエストの連想配列
     * @return string 発行先から戻されたレスポンス文字列
     */
    function __execCurl($header, $request)
    {
    	//
		$ch = curl_init();
		//
		$submit_url = $this->_submit_url;
		$user_agent = $this->_user_agent;
		//
		curl_setopt($ch, CURLOPT_URL, $submit_url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		curl_setopt($ch, CURLOPT_USERAGENT, $user_agent);
		curl_setopt($ch, CURLOPT_HEADER, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 45);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $request);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,  2);
		curl_setopt($ch, CURLOPT_FORBID_REUSE, true);
		curl_setopt($ch, CURLOPT_POST, 1);
		//
		$res = curl_exec($ch);
error_log(print_r($res, true));
//		$res  = 'RESULT=0&RESPMSG=GEMA&PNREF=1234567890';
		//
		curl_close($ch);
		//
		return $res;
    }

}

/* End of file */